package com.myketiab;

import android.content.IntentFilter;
import android.util.Log;

import com.facebook.react.ReactActivity;
import com.facebook.react.bridge.Callback;
import com.google.gson.Gson;
import com.myketiab.util.IabBroadcastReceiver;
import com.myketiab.util.IabHelper;
import com.myketiab.util.IabResult;
import com.myketiab.util.Inventory;
import com.myketiab.util.Purchase;


public class MyketIap {

    private static MyketIap instance;

    // Debug tag, for logging
    static final String TAG = "TrivialDrive";

    // SKUs for our products: the premium upgrade (non-consumable) and gas (consumable)
    static final String SKU_PREMIUM = "premium";
    static final String SKU_GAS = "gas";

    // SKU for our subscription (infinite gas)
    static final String SKU_INFINITE_GAS_MONTHLY = "infinite_gas_monthly";
    static final String SKU_INFINITE_GAS_YEARLY = "infinite_gas_yearly";

    // (arbitrary) request code for the purchase flow
    static final int RC_REQUEST = 10001;

    // React Bridge Callbacks
    Callback reactOnGotInventoryFinishedListener;
    Callback reactOnConsumeFinishedListener;
    Callback reactOnPurchaseFinishedListener;
    Callback reactOnOnReceivedListener;

    // Gson
    Gson gson = new Gson();


    public static MyketIap getInstance(ReactActivity activity) {
        if (instance == null)
            instance = new MyketIap(activity);
        return instance;
    }

    public static MyketIap getInstance() {
        if (instance == null)
            throw new NullPointerException("The activity not given to MyketIap.");
        return instance;
    }

    // Main Activity
    ReactActivity activity;

    // The helper object
    private IabHelper mHelper;

    // Provides purchase notification while this app is running
    private IabBroadcastReceiver mBroadcastReceiver;

    private MyketIap(ReactActivity activity) {
        this.activity = activity;
    }


    public void initializeHelper(Callback gotInventoryListener, Callback broadcastOnReceived) {
        /* base64EncodedPublicKey should be YOUR APPLICATION'S PUBLIC KEY
         * (that you got from the Google Play developer console). This is not your
         * developer public key, it's the *app-specific* public key.
         *
         * Instead of just storing the entire literal string here embedded in the
         * program,  construct the key at runtime from pieces or
         * use bit manipulation (for example, XOR with some other string) to hide
         * the actual key.  The key itself is not secret information, but we don't
         * want to make it easy for an attacker to replace the public key with one
         * of their own and then fake messages from the server.
         */
        String base64EncodedPublicKey = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCh76YhbzdYXgnMaZyFqMO7SnkKd0NPTEE3paBj/rOnu5EUdEUaPcxPiLUn/mxbYu9TZ1nmAF2JMUTg0G+6/wA0WLbJJsXFFoR01kyXp43/fA+wd+/zOTVgNLqj7ja7ZKQIAiFDttzWE7O9CzgpLMVVGw1ldM67HGy1z4VWFHeSiwIDAQAB";

        // Some sanity checks to see if the developer (that's you!) really followed the
        // instructions to run this sample (don't put these checks on your app!)
        if (base64EncodedPublicKey.contains("CONSTRUCT_YOUR")) {
            throw new RuntimeException("Please put your app's public key in MainActivity.java. See README.");
        }

        // Create the helper, passing it our context and the public key to verify signatures with
        Log.d(TAG, "Creating IAB helper.");
        mHelper = new IabHelper(activity, base64EncodedPublicKey);
        this.reactOnGotInventoryFinishedListener = gotInventoryListener;
        this.reactOnOnReceivedListener = broadcastOnReceived;

        // enable debug logging (for a production application, you should set this to false).
        mHelper.enableDebugLogging(true);

        // Start setup. This is asynchronous and the specified listener
        // will be called once setup completes.
        Log.d(TAG, "Starting setup.");
        mHelper.startSetup(new IabHelper.OnIabSetupFinishedListener() {
            public void onIabSetupFinished(IabResult result) {
                Log.d(TAG, "Setup finished.");

                if (!result.isSuccess()) {
                    // Oh noes, there was a problem.
                    Log.e(TAG, result.getMessage());
                    return;
                }

                // Have we been disposed of in the meantime? If so, quit.
                if (mHelper == null) return;

                // Important: Dynamically register for broadcast messages about updated purchases.
                // We register the receiver here instead of as a <receiver> in the Manifest
                // because we always call getPurchases() at startup, so therefore we can ignore
                // any broadcasts sent while the app isn't running.
                // Note: registering this listener in an Activity is a bad idea, but is done here
                // because this is a SAMPLE. Regardless, the receiver must be registered after
                // IabHelper is setup, but before first call to getPurchases().

                mBroadcastReceiver = new IabBroadcastReceiver(mBroadcastListener);
                IntentFilter broadcastFilter = new IntentFilter(IabBroadcastReceiver.ACTION);
                activity.registerReceiver(mBroadcastReceiver, broadcastFilter);

                // IAB is fully set up. Now, let's get an inventory of stuff we own.
                Log.d(TAG, "Setup successful. Querying inventory.");
                mHelper.queryInventoryAsync(mGotInventoryListener);
            }
        });
    }


    public void launchPurchaseFlow(String skuString, int requestCode, Callback reactOnPurchaseFinishedListener) {
        Log.d(TAG, "Buy gas button clicked.");
        Log.d(TAG, "Launching purchase flow for gas.");

        String payload = "";

        this.reactOnPurchaseFinishedListener = reactOnPurchaseFinishedListener;

        mHelper.launchPurchaseFlow(activity, skuString, requestCode, mPurchaseFinishedListener, payload);
    }

    public void consumeAsync(String sku, Callback reactConsumeListener) {
        this.reactOnConsumeFinishedListener = reactConsumeListener;
        Log.d(TAG, "Thats is : " + sku + "");
        Log.d(TAG, "Thats is : " + gson.fromJson(sku, Purchase.class) + "");
        mHelper.consumeAsync(gson.fromJson(sku, Purchase.class), mConsumeFinishedListener);
    }

    // Listener that's called when we finish querying the items and subscriptions we own
    IabHelper.QueryInventoryFinishedListener mGotInventoryListener = new IabHelper.QueryInventoryFinishedListener() {
        public void onQueryInventoryFinished(IabResult result, Inventory inventory) {
            Log.d(TAG, "Query inventory finished.");

            // Have we been disposed of in the meantime? If so, quit.
            if (mHelper == null) return;

            /*
             * Check for items we own. Notice that for each purchase, we check
             * the developer payload to see if it's correct! See
             * verifyDeveloperPayload().
             */
            reactOnGotInventoryFinishedListener.invoke(gson.toJson(result), gson.toJson(inventory));
        }
    };

    IabHelper.OnIabPurchaseFinishedListener mPurchaseFinishedListener = new IabHelper.OnIabPurchaseFinishedListener() {
        public void onIabPurchaseFinished(IabResult result, Purchase purchase) {
            Log.d(TAG, "Purchase finished: " + result + ", purchase: " + purchase);

            // if we were disposed of in the meantime, quit.
            if (mHelper == null) return;

            reactOnPurchaseFinishedListener.invoke(gson.toJson(result), gson.toJson(purchase));

        }
    };


    // Called when consumption is complete
    IabHelper.OnConsumeFinishedListener mConsumeFinishedListener = new IabHelper.OnConsumeFinishedListener() {
        public void onConsumeFinished(Purchase purchase, IabResult result) {
            Log.d(TAG, "Consumption finished. Purchase: " + purchase + ", result: " + result);

            // if we were disposed of in the meantime, quit.
            if (mHelper == null) return;

            // We know this is the "gas" sku because it's the only one we consume,
            // so we don't check which sku was consumed. If you have more than one
            // sku, you probably should check...
            reactOnConsumeFinishedListener.invoke(gson.toJson(purchase), gson.toJson(result));

            Log.d(TAG, "End consumption flow.");
        }
    };

    IabBroadcastReceiver.IabBroadcastListener mBroadcastListener = new IabBroadcastReceiver.IabBroadcastListener() {
        @Override
        public void receivedBroadcast() {
            if (reactOnOnReceivedListener != null) {
                Log.d(TAG, "Broadcast Receiver OnReceived() !");
                reactOnOnReceivedListener.invoke();
            }
        }
    };

    public IabHelper getHelper() {
        return mHelper;
    }

    public void setHelper(IabHelper mHelper) {
        this.mHelper = mHelper;
    }

    public IabBroadcastReceiver getBroadcastReceiver() {
        return mBroadcastReceiver;
    }
}
